"""
A summary of interesting image APIs to https://bbs.mihoyo.com/ys/.
"""


from ast import Pass


APIS = {
    'icon': {
        32: 'https://ys.mihoyo.com/main/favicon.ico',
        256: 'https://genshin.mihoyo.com/favicon.ico',
    },
    'bg': 'https://ys.mihoyo.com/',
    'cos': {

        'hot': 'https://bbs.mihoyo.com/ys/home/49?type=hot',
        'latest_reply': 'https://bbs.mihoyo.com/ys/home/49?type=1',
        'latest_post': 'https://bbs.mihoyo.com/ys/home/49?type=2',
        'rank': {
            'daily': 'https://bbs.mihoyo.com/ys/imgRanking/49/1',
            'weekly': 'https://bbs.mihoyo.com/ys/imgRanking/49/2',
            'monthly': 'https://bbs.mihoyo.com/ys/imgRanking/49/3',
            'limit_length': 20
        },
    },
    'fan': {
        'hot': 'https://bbs.mihoyo.com/ys/home/29?type=hot',
        'latest_reply': 'https://bbs.mihoyo.com/ys/home/29?type=1',
        'latest_post': 'https://bbs.mihoyo.com/ys/home/29?type=2',
        'rank': {
            'daily': 'https://bbs.mihoyo.com/ys/imgRanking/29/1',
            'weekly': 'https://bbs.mihoyo.com/ys/imgRanking/29/2',
            'monthly': 'https://bbs.mihoyo.com/ys/imgRanking/29/3',
            'limit_length': 100
        },
    },
    'topic': { # TODO (GitHub@XavierJiezou): Download images belonging to topic tags.
        'emoji': {
            'new': 'https://bbs.mihoyo.com/ys/topicDetail/105',
            'select': 'https://bbs.mihoyo.com/ys/topicDetail/105?type=1',
            'hot': 'https://bbs.mihoyo.com/ys/topicDetail/105?type=2',
        },
        'wallpaper': {
            'new': 'https://bbs.mihoyo.com/ys/topicDetail/558',
            'hot': 'https://bbs.mihoyo.com/ys/topicDetail/558?type=2',
        },
    },
    'css_selector': {
        'article': {
            'url': 'div.mhy-article-list__body>div>a.mhy-router-link',
            'title': 'div.mhy-article-page__title>h1',
            'image': 'div.mhy-img-article img',
        },
        'topicDetail': {
            'url': '',
            'title': '',
            'image': '',
        },
    },
}


if __name__ == '__main__':
    pass



import json
import os
import re
import time
from typing import Any

import fire
from pytest import param
from rich.console import Console
from rich.progress import (
    Progress,
    SpinnerColumn,
    MofNCompleteColumn,
    TimeElapsedColumn,
)
from rich.prompt import Prompt, IntPrompt
from rich.theme import Theme
from selenium import webdriver

from ys_dl.core import YsDl
from ys_dl.apis import APIS


class YsDlCli(object):

    def __init__(self) -> None:
        self.options = webdriver.ChromeOptions()
        self.options.add_experimental_option(
            'excludeSwitches',
            ['enable-logging']
        )
        theme = Theme({
            "success": "cyan",
            "failure": "b red",
        })
        self.console = Console(theme=theme)
        self.progress = Progress(
            SpinnerColumn(),
            MofNCompleteColumn(),
            *Progress.get_default_columns(),
            TimeElapsedColumn(),
            console=self.console,
        )

    @property
    def version(self) -> str:
        return YsDl().version

    def icon(self) -> None:
        size = Prompt.ask(
            prompt='Select the size',
            choices=['256', '32'],
            default='256'
        )
        result = YsDl().get_icon_img(size=int(size))
        if result['status'] == 200:
            with self.console.status(f'Downloading {size}x{size} icon to local'):
                YsDl().dl_img(result['data'])
        else:
            pass
        self.console.print_json(json.dumps(result))

    def bg(self) -> None:
        result = YsDl().get_bg_img()
        if result['status'] == 200:
            with self.console.status('Downloading background image to local'):
                YsDl().dl_img(result['data'], img_name='bg.jpg')
        else:
            pass
        self.console.print_json(json.dumps(result))

    def _get_article_label(self) -> str:
        labels = ['rank', 'hot', 'latest_post', 'latest_reply']
        emojis = [
            ':rocket:',
            ':fire:',
            ':envelope_with_arrow:',
            ':loudspeaker:',
        ]
        tmp_str = '/'.join([f'{e} {l}' for e, l in zip(emojis, labels)])
        return Prompt.ask(
            prompt=f'Enter the label [b][magenta][{tmp_str}][/magenta][/b]',
            choices=labels,
            show_choices=False,
            default='rank',
        )

    def _get_rank_key(self) -> str:
        return Prompt.ask(
            prompt='Select the key',
            choices=['daily', 'weekly', 'monthly'],
            default='daily',
        )

    def _get_rank_api(self) -> str:
        return Prompt.ask(
            prompt='Select the api',
            choices=['cos', 'fan'],
            default='cos',
        )

    def _get_dl_limit(self, max_value: int, default_value: int) -> int:
        while True:
            limit = IntPrompt.ask(
                prompt=f'Enter the number of pages to download [b][magenta][0~{max_value}][/magenta][/b]',
                default=default_value,
            )
            if 0 <= limit <= max_value:
                break
            self.console.print(
                f':pile_of_poo: [prompt.invalid]Number must be between 0 and {max_value}')
        return limit

    def _dl_img_loop(
        self,
        article_urls: list[str],
        driver: webdriver,
        is_rank: bool,
        params: list[Any],
    ) -> None:
        if is_rank:
            overall_desc = f'{params[1]}/{params[0]}/{params[2]}'
        else:
            if params[1] == 'rank':
                overall_desc = '/'.join(params[:3])
            else:
                overall_desc = '/'.join(params[:2])
        with self.progress:
            task = self.progress.add_task(
                description=f'[green]{overall_desc}',
                total=len(article_urls),
            )
            count = 0
            for idx, url in enumerate(article_urls):
                image = YsDl().get_article_img(url, driver)
                title = image['data']['title'].replace(' ', '')
                title = re.sub('[/\\:*?"<>|]', '', title)
                urls = image['data']['urls']
                save_dir = os.path.join(
                    f'{overall_desc}/',
                    f'[{idx+1:0>{len(str(len(article_urls)))}}]{title}/',
                )
                # if len(title)<=10:
                #     _title = title
                # else:
                #     _title = f'{title[:7]}...'
                sub_task = self.progress.add_task(
                    description=f'[cyan][{idx+1:0>{len(str(len(article_urls)))}}]{title}',
                    total=len(urls),
                )
                for _idx, _url in enumerate(urls):
                    YsDl().dl_img(
                        img_url=_url,
                        save_dir=save_dir,
                        img_name=f'{_idx+1}.jpg',
                    )
                    save_path = os.path.join(save_dir, f'{_idx+1}.jpg')
                    link_path = os.path.join(os.getcwd(), save_dir, f'{_idx+1}.jpg')
                    self.console.log(f'Downloaded {save_path}', style=f'link {link_path}')
                    self.progress.update(sub_task, advance=1)
                    time.sleep(1)
                    count += 1
                self.progress.update(task, advance=1)
        with self.console.status('Quitting the webdriver'):
            driver.quit()
        self.console.print(f'[green]{count} images downloaded, done!')

    def _get_params(self, api_key: str, is_rank: bool) -> list[Any]:
        if is_rank:
            sub_key = 'rank'
            rank_key = self._get_rank_key()
            max_value = APIS[api_key][sub_key]['limit_length']
            url_nums = self._get_dl_limit(max_value, max_value)
        else:
            label = self._get_article_label()
            if label == 'rank':
                sub_key = 'rank'
                rank_key = self._get_rank_key()
                max_value = APIS[api_key][sub_key]['limit_length']
                url_nums = self._get_dl_limit(max_value, max_value)
            else:
                sub_key = label
                rank_key = None
                max_value = 500
                url_nums = self._get_dl_limit(max_value, 100)
        return [
            api_key,
            sub_key,
            rank_key,
            url_nums,
        ]

    def _article(self, api_key: str = 'cos', is_rank: bool = False) -> None:
        with self.console.status('Yuanshen downloader booting up'):
            time.sleep(1)
        with self.console.status('Creating a webdriver (headless)'):
            self.options.add_argument('--headless')
            driver = webdriver.Chrome(options=self.options)
        params = self._get_params(api_key, is_rank)
        with self.console.status('Getting urls to articles'):
            result = YsDl().get_article_url(
                driver,
                *params,
            )
        self._dl_img_loop(result['data'], driver, is_rank, params)

    def cos(self) -> None:
        self._article('cos')

    def fan(self) -> None:
        self._article('fan')

    def rank(self) -> None:
        api_key = self._get_rank_api()
        self._article(api_key, is_rank=True)

    # TODO (GitHub@XavierJiezou): Download images belonging to topic tags.
    def topic(self) -> str:
        return 'TODO (GitHub@XavierJiezou): Download images belonging to topic tags.'


def main():
    fire.Fire(YsDlCli)


if __name__ == '__main__':
    main()



import os
import re
import time
from typing import Any

from fake_useragent import UserAgent
import pyautogui
import pyperclip
import requests
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from ys_dl.apis import APIS
from ys_dl import __version__


class YsDl(object):

    @property
    def version(self):
        return __version__

    def dl_img_static(
        self,
        img_url: str,
        save_path: str,
    ) -> dict[str, Any]:
        headers = {'user-agent': UserAgent().random}
        res = requests.get(img_url, headers=headers)
        if res.status_code == 200:
            with open(save_path, 'wb') as f:
                f.write(res.content)
            return {
                'status': res.status_code,
                'data': save_path,
                'msg': 'success',
            }
        else:
            return {
                'status': res.status_code,
                'data': None,
                'msg': 'failure',
            }

    def dl_img_dynamic(
        self,
        img_url: str,
        save_path: str,
        driver: webdriver,
    ) -> dict[str, Any]:
        driver.get(img_url)
        driver.implicitly_wait(5)
        pyautogui.hotkey('ctrlleft', 's')
        time.sleep(1)
        pyperclip.copy(save_path)
        time.sleep(0.5)
        pyautogui.hotkey('ctrlleft', 'v')
        time.sleep(0.5)
        pyautogui.press('enter')
        time.sleep(1)
        if os.path.exists(save_path):
            return {
                'status': 200,
                'data': save_path,
                'msg': 'success',
            }
        else:
            return {
                'status': 404,
                'data': None,
                'msg': 'failure',
            }

    def dl_img(
        self,
        img_url: str,
        mode: str = 'static',
        save_dir: str = None,
        img_name: str = None,
        driver: webdriver = None,
    ) -> dict[str, Any]:
        if not save_dir:
            save_dir = os.getcwd()
        else:
            os.makedirs(save_dir, exist_ok=True)
        if not img_name:
            img_name = os.path.basename(img_url)
        else:
            pass
        save_path = os.path.join(save_dir, img_name)
        if os.path.exists(save_path):
            return {
                'status': 200,
                'data': save_path,
                'msg': 'success',
            }
        else:
            if mode == 'static':
                return self.dl_img_static(img_url, save_path)
            else:
                return self.dl_img_dynamic(img_url, save_path, driver)

    def get_icon_img(
        self,
        api_key: str = 'icon',
        size: int = 256,
    ) -> dict[str, Any]:
        url = APIS.get(api_key).get(size)
        return {
            'status': 200,
            'data': url,
            'msg': 'success'
        }

    def get_bg_img(self, api_key: str = 'bg') -> dict[str, Any]:
        url = APIS.get(api_key)
        headers = {'user-agent': UserAgent().random}
        res = requests.get(url=url, headers=headers)
        if res.status_code == 200:
            pattern = re.compile('https://web.*?config_.*?.js')
            js_url = re.findall(pattern, res.text)[0]
            js_res = requests.get(url=js_url, headers=headers)
            if js_res.status_code == 200:
                pattern = re.compile('resources":{"(.*?.jpg)"')
                bg_url = re.findall(pattern, js_res.text)[0]
                return {
                    'status': js_res.status_code,
                    'data': bg_url,
                    'msg': 'success'
                }
            else:
                return {
                    'status': js_res.status_code,
                    'data': None,
                    'msg': 'failure'
                }
        else:
            return {
                'status': res.status_code,
                'data': None,
                'msg': 'failure'
            }

    def get_article_url(
        self,
        driver: webdriver,
        api_key: str = 'cos',
        sub_key: str = 'rank',
        rank_key: str = 'daily',
        url_nums: int = 100,
        max_trying: int = 100,
    ) -> dict[str, Any]:
        url = APIS[api_key][sub_key]
        if sub_key == 'rank':
            url_nums = min(url['limit_length'], url_nums)
            url = url[rank_key]
        else:
            pass
        driver.maximize_window()
        driver.get(url)
        driver.implicitly_wait(5)
        action = ActionChains(driver)
        count = 0
        for _ in range(max_trying):
            tag_a = driver.find_elements(
                By.CSS_SELECTOR,
                APIS['css_selector']['article']['url'],
            )
            if len(tag_a) >= url_nums:
                break
            else:
                action.send_keys(Keys.PAGE_DOWN).perform()
                time.sleep(0.5)
                count += 1
        if count < max_trying:
            urls = []
            for _ in tag_a[:url_nums]:
                urls.append(_.get_attribute('href'))
            return {
                'status': 200,
                'data': urls,
                'msg': 'success',
            }
        else:
            return {
                'status': 404,
                'data': None,
                'msg': 'failure'
            }

    def get_article_img(
        self,
        article_url: str,
        driver: webdriver,
        size: str = 'large'
    ) -> dict[str, Any]:
        driver.maximize_window()
        driver.get(article_url)
        driver.implicitly_wait(5)
        value = APIS['css_selector']['article']['title']
        title = driver.find_element(By.CSS_SELECTOR, value).text
        value = APIS['css_selector']['article']['image']
        image = driver.find_elements(By.CSS_SELECTOR, value)
        if image:
            if size == 'large':
                urls = [_.get_attribute('large') for _ in image]
            else:
                urls = [_.get_attribute('src') for _ in image]
            return {
                'status': 200,
                'data': {
                    'title': title,
                    'urls': urls,
                },
                'msg': 'success'
            }
        else:
            return {
                'status': 404,
                'data': None,
                'msg': 'failure'
            }

    # TODO (GitHub@XavierJiezou): Download images belonging to topic tags.
    def get_topic_url(self):
        pass

    # TODO (GitHub@XavierJiezou): Download images belonging to topic tags.
    def get_topic_img(self):
        pass

    # TODO (GitHub@XavierJiezou): Download images belonging to topic tags.
    def get_topic(self):
        pass


if __name__ == '__main__':
    pass
